@@ -1,4 +1,4 @@
-;;; ocamldebug.el --- Run ocamldebug / camldebug under Emacs.
+;;; ocamldebug.el --- Run ocamldebug / camldebug under Emacs  -*- lexical-binding:t -*-
 ;; Derived from gdb.el.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -36,12 +36,12 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
 (require 'comint)
 (require 'shell)
-(require 'tuareg (expand-file-name
-                  "tuareg" (file-name-directory (or load-file-name
-                                                    byte-compile-current-file))))
+(require 'tuareg (expand-file-name "tuareg" (file-name-directory
+                                             (or load-file-name
+                                                 byte-compile-current-file
+                                                 buffer-file-name))))
 (require 'derived)
 
 ;;; Variables.
@@ -63,31 +63,35 @@
 (defvar ocamldebug-prompt-pattern "^(\\(ocd\\|cdb\\)) *"
   "A regexp to recognize the prompt for ocamldebug.")
 
-(defvar ocamldebug-overlay-event nil
-  "Overlay for displaying the current event.")
-(defvar ocamldebug-overlay-under nil
-  "Overlay for displaying the current event.")
-(defvar ocamldebug-event-marker nil
+(defvar ocamldebug-overlay-event
+  (let ((ol (make-overlay (point) (point))))
+    (overlay-put ol 'face 'ocamldebug-event)
+    (delete-overlay ol) ;; Disconnect it from current buffer.
+    ol)
+  "Overlay for displaying the first/last char of current event.")
+(defvar ocamldebug-overlay-under
+  (let ((ol (make-overlay (point) (point))))
+    (overlay-put ol 'face 'ocamldebug-underline)
+    (delete-overlay ol) ;; Disconnect it from current buffer.
+    ol)
+  "Overlay for displaying the rest of current event.")
+(defvar ocamldebug-event-marker (make-marker)
   "Marker for displaying the current event.")
 
 (defvar ocamldebug-track-frame t
-  "*If non-nil, always display current frame position in another window.")
+  "If non-nil, always display current frame position in another window.")
 
-(cond
- ((and (fboundp 'make-overlay) window-system)
-  (make-face 'ocamldebug-event)
-  (make-face 'ocamldebug-underline)
-  (unless (face-differs-from-default-p 'ocamldebug-event)
-    (invert-face 'ocamldebug-event))
-  (unless (face-differs-from-default-p 'ocamldebug-underline)
-    (set-face-underline 'ocamldebug-underline t))
-  (setq ocamldebug-overlay-event (make-overlay 1 1))
-  (overlay-put ocamldebug-overlay-event 'face 'ocamldebug-event)
-  (setq ocamldebug-overlay-under (make-overlay 1 1))
-  (overlay-put ocamldebug-overlay-under 'face 'ocamldebug-underline))
- (t
-  (setq ocamldebug-event-marker (make-marker))
-  (setq overlay-arrow-string "=>")))
+(defface ocamldebug-event
+  '((t :invert t))
+  "Face to highlight the first/last char of current event."
+  :group 'tuareg)
+
+(defface ocamldebug-underline
+  ;; FIXME: The name should describe what it's used for, not what it looks
+  ;; like by default!
+  '((t :underline t))
+  "Face to highlight the rest of current event."
+  :group 'tuareg)
 
 ;;; OCamldebug mode.
 
@@ -99,17 +103,16 @@
 (defvar ocamldebug-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map "\C-c" ocamldebug-prefix-map)
-    (define-key map "\C-l" 'ocamldebug-refresh)
+    (define-key map "\C-l" #'ocamldebug-refresh)
     ;; This is already the default anyway!
     ;;(define-key map "\t" 'comint-dynamic-complete)
     (define-key map "\M-?"
       ;; FIXME: This binding is wrong since comint-dynamic-list-completions
       ;; is a function, not a command.
-      'comint-dynamic-list-completions)
+      #'comint-dynamic-list-completions)
     map))
 
 (define-derived-mode ocamldebug-mode comint-mode "OCaml-Debugger"
-
   "Major mode for interacting with an ocamldebug process.
 
 The following commands are available:
@@ -138,15 +141,12 @@ C-x SPACE sets break point at current line."
   (setq-local ocamldebug-filter-accumulator "")
   (setq-local ocamldebug-filter-function #'ocamldebug-marker-filter)
   (setq-local comint-prompt-regexp ocamldebug-prompt-pattern)
-  (setq-local comint-dynamic-complete-functions
-       (cons (if (boundp 'completion-at-point-functions)
-                 #'ocamldebug-capf #'ocamldebug-complete)
-             comint-dynamic-complete-functions))
+  (add-hook 'comint-dynamic-complete-functions #'ocamldebug-capf nil 'local)
   (setq-local comint-prompt-read-only t)
   (setq-local paragraph-start comint-prompt-regexp)
   (setq-local ocamldebug-last-frame-displayed-p t)
   (setq-local shell-dirtrackp t)
-  (add-hook 'comint-input-filter-functions 'shell-directory-tracker nil t))
+  (add-hook 'comint-input-filter-functions #'shell-directory-tracker nil t))
 
 ;;; Keymaps.
 
@@ -184,7 +184,7 @@ representation is simply concatenated with the COMMAND."
                (interactive "P")
                (ocamldebug-call ,name ,args
                                 (ocamldebug-numeric-arg arg))))
-       (define-key ocamldebug-prefix-map ,key ',fun))))
+       (define-key ocamldebug-prefix-map ,key #',fun))))
 
 (def-ocamldebug "step"	"\C-s"	"Step one source line with display.")
 (def-ocamldebug "run"	"\C-r"	"Run the program.")
@@ -202,11 +202,10 @@ representation is simply concatenated with the COMMAND."
   "@ \"%m\" # %c")
 
 (defun ocamldebug-kill-filter (string)
-  ;gob up stupid questions :-)
+  ;; Gob up stupid questions :-)
   (setq ocamldebug-filter-accumulator
 	(concat ocamldebug-filter-accumulator string))
-  (when (string-match "\\(.* \\)(y or n) "
-                      ocamldebug-filter-accumulator)
+  (when (string-match "\\(.* \\)(y or n) " ocamldebug-filter-accumulator)
     (setq ocamldebug-kill-output
 	  (cons t (match-string 1 ocamldebug-filter-accumulator)))
     (setq ocamldebug-filter-accumulator ""))
@@ -225,7 +224,7 @@ representation is simply concatenated with the COMMAND."
 (defun ocamldebug-kill ()
   "Kill the program."
   (interactive)
-  (let ((ocamldebug-kill-output))
+  (let (ocamldebug-kill-output)
     (with-current-buffer ocamldebug-current-buffer
       (let ((proc (get-buffer-process (current-buffer)))
 	    (ocamldebug-filter-function #'ocamldebug-kill-filter))
@@ -235,30 +234,37 @@ representation is simply concatenated with the COMMAND."
 	  (accept-process-output proc))))
     (if (not (car ocamldebug-kill-output))
 	(error (cdr ocamldebug-kill-output))
-      (sit-for 0 300)
+      (sit-for 0.3)
       (ocamldebug-call-1 (if (y-or-n-p (cdr ocamldebug-kill-output)) "y" "n")))))
 ;;FIXME: ocamldebug doesn't output the Hide marker on kill
 
 (defun ocamldebug-goto-filter (string)
-  ;accumulate onto previous output
+  ;; Accumulate onto previous output
   (setq ocamldebug-filter-accumulator
 	(concat ocamldebug-filter-accumulator string))
-  (when (or (string-match (concat
-                           "\\(\n\\|\\`\\)[ \t]*\\([0-9]+\\)[ \t]+"
-                           ocamldebug-goto-position
-                           "-[0-9]+[ \t]*\\(before\\).*\n")
-                          ocamldebug-filter-accumulator)
-            (string-match (concat
-                           "\\(\n\\|\\`\\)[ \t]*\\([0-9]+\\)[ \t]+[0-9]+-"
-                           ocamldebug-goto-position
-                           "[ \t]*\\(after\\).*\n")
-                          ocamldebug-filter-accumulator))
-    (setq ocamldebug-goto-output
-	  (match-string 2 ocamldebug-filter-accumulator))
+  ;;    Address  Characters        Kind      Repr.
+  ;;     14452     64-82      before/fun
+  ;;     14584    182-217      after/ret
+  ;;0:     30248     -1--1          pseudo
+  ;;0:     30076     64-82      before/fun
+  (when (or (string-match
+             (concat "\\(?:\n\\|\\`\\)[ \t]*"
+                     "\\([0-9]+\\)\\(?::[ \t]*\\([0-9]+\\)\\)?[ \t]+"
+                     ocamldebug-goto-position
+                     "-[0-9]+[ \t]*before.*\n")
+             ocamldebug-filter-accumulator)
+            (string-match
+             (concat "\\(?:\n\\|\\`\\)[ \t]*"
+                     "\\([0-9]+\\)\\(?::[ \t]*\\([0-9]+\\)\\)?[ \t]+[0-9]+-"
+                     ocamldebug-goto-position
+                     "[ \t]*after.*\n")
+             ocamldebug-filter-accumulator))
+    (let ((id (match-string 1 ocamldebug-filter-accumulator))
+          (pos (match-string 2 ocamldebug-filter-accumulator)))
+      (setq ocamldebug-goto-output (if pos (concat id ":" pos) id)))
     (setq ocamldebug-filter-accumulator
 	  (substring ocamldebug-filter-accumulator (1- (match-end 0)))))
-  (when (string-match comint-prompt-regexp
-                      ocamldebug-filter-accumulator)
+  (when (string-match comint-prompt-regexp ocamldebug-filter-accumulator)
     (setq ocamldebug-goto-output (or ocamldebug-goto-output 'fail))
     (setq ocamldebug-filter-accumulator ""))
   (when (string-match "\n\\(.*\\)\\'" ocamldebug-filter-accumulator)
@@ -268,17 +274,15 @@ representation is simply concatenated with the COMMAND."
 
 (def-ocamldebug "goto" "\C-g")
 (defun ocamldebug-goto (&optional time)
-
   "Go to the execution time TIME.
 
 Without TIME, the command behaves as follows: In the ocamldebug buffer,
-if the point at buffer end, goto time 0\; otherwise, try to obtain the
+if the point at buffer end, goto time 0; otherwise, try to obtain the
 time from context around point.  In an OCaml buffer, try to find the
 time associated in execution history with the current point location.
 
 With a negative TIME, move that many lines backward in the ocamldebug
 buffer, then try to obtain the time from context around point."
-
   (interactive "P")
   (cond
    (time
@@ -287,64 +291,70 @@ buffer, then try to obtain the time from context around point."
 	(save-selected-window
 	  (select-window (get-buffer-window ocamldebug-current-buffer))
 	  (save-excursion
-	    (if (re-search-backward "^Time : [0-9]+ - pc : [0-9]+ "
-				    nil t (- 1 ntime))
-		(ocamldebug-goto nil)
-	      (error "I don't have %d times in my history"
-		     (- 1 ntime))))))))
+            (if (re-search-backward
+                 "^Time *: [0-9]+ - pc *: [0-9]+\\(?::[0-9]+\\)? "
+                 nil t (- 1 ntime))
+                (ocamldebug-goto nil)
+              (error "I don't have %d times in my history"
+                     (- 1 ntime))))))))
    ((eq (current-buffer) ocamldebug-current-buffer)
-      (let ((time (cond
-		   ((eobp) 0)
-		   ((save-excursion
-		      (beginning-of-line 1)
-		      (looking-at "^Time : \\([0-9]+\\) - pc : [0-9]+ "))
-		    (string-to-number (match-string 1)))
-		   ((string-to-number (ocamldebug-format-command "%e"))))))
-	(ocamldebug-call "goto" nil time)))
+    (let ((time (cond
+                 ((eobp) 0)
+                 ((save-excursion
+                    (beginning-of-line 1)
+                    (looking-at
+                     "^Time *: \\([0-9]+\\) - pc *: [0-9]+\\(?::[0-9]+\\)? "))
+                  (string-to-number (match-string 1)))
+                 ((string-to-number (ocamldebug-format-command "%e"))))))
+      (ocamldebug-call "goto" nil time)))
    (t
     (let ((module (ocamldebug-module-name (buffer-file-name)))
-	  (ocamldebug-goto-position (int-to-string (1- (point))))
-	  (ocamldebug-goto-output) (address))
-      ;get a list of all events in the current module
+          (ocamldebug-goto-position (int-to-string (1- (point))))
+          ocamldebug-goto-output address)
+      ;; Get a list of all events in the current module
       (with-current-buffer ocamldebug-current-buffer
-	(let* ((proc (get-buffer-process (current-buffer)))
-	       (ocamldebug-filter-function #'ocamldebug-goto-filter))
-	  (ocamldebug-call-1 (concat "info events " module))
-	  (while (not (and ocamldebug-goto-output
-		      (zerop (length ocamldebug-filter-accumulator))))
-	    (accept-process-output proc))
-	  (setq address (unless (eq ocamldebug-goto-output 'fail)
-			  (re-search-backward
-			   (concat "^Time : \\([0-9]+\\) - pc : "
-				   ocamldebug-goto-output
-				   " - module "
-				   module "$")
+        (let* ((proc (get-buffer-process (current-buffer)))
+               (ocamldebug-filter-function #'ocamldebug-goto-filter))
+          (ocamldebug-call-1 (concat "info events " module))
+          (while (not (and ocamldebug-goto-output
+                           (zerop (length ocamldebug-filter-accumulator))))
+            (accept-process-output proc))
+          (setq address (unless (eq ocamldebug-goto-output 'fail)
+                          (re-search-backward
+                           (concat "^Time *: \\([0-9]+\\) - pc *: "
+                                   ocamldebug-goto-output
+                                   " - module "
+                                   module "$")
                            nil t)
-			  (match-string 1)))))
+                          (match-string 1)))))
       (if address (ocamldebug-call "goto" nil (string-to-number address))
-	(error "No time at %s at %s" module ocamldebug-goto-position))))))
+        (error "No time at %s at %s" module ocamldebug-goto-position))))))
 
 
 (defun ocamldebug-delete-filter (string)
   (setq ocamldebug-filter-accumulator
-	(concat ocamldebug-filter-accumulator string))
+        (concat ocamldebug-filter-accumulator string))
   (when (string-match
-         (concat "\\(\n\\|\\`\\)[ \t]*\\([0-9]+\\)[ \t]+[0-9]+[ \t]*in "
+         ;; Num    Address  Where
+         ;;  1      14552  file u.ml, line 5, characters 1-34
+         ;;  1 0:     30176  file u.ml, line 5, characters 1-34
+         (concat "\\(?:\n\\|\\`\\)[ \t]*\\([0-9]+\\)[ \t]+"
+                 "[0-9]+\\(?::[ \t]*[0-9]+\\)?[ \t]+file +"
                  (regexp-quote ocamldebug-delete-file)
                  ", character "
                  ocamldebug-delete-position "\n")
          ocamldebug-filter-accumulator)
     (setq ocamldebug-delete-output
-	  (match-string 2 ocamldebug-filter-accumulator))
+          (match-string 1 ocamldebug-filter-accumulator))
     (setq ocamldebug-filter-accumulator
-	  (substring ocamldebug-filter-accumulator (1- (match-end 0)))))
+          (substring ocamldebug-filter-accumulator (1- (match-end 0)))))
   (when (string-match comint-prompt-regexp
                       ocamldebug-filter-accumulator)
     (setq ocamldebug-delete-output (or ocamldebug-delete-output 'fail))
     (setq ocamldebug-filter-accumulator ""))
   (if (string-match "\n\\(.*\\)\\'" ocamldebug-filter-accumulator)
       (setq ocamldebug-filter-accumulator
-	    (match-string 1 ocamldebug-filter-accumulator)))
+            (match-string 1 ocamldebug-filter-accumulator)))
   "")
 
 
@@ -360,20 +370,21 @@ try to find the breakpoint associated with the current point location.
 With a negative ARG, look for the -ARGth breakpoint pattern in the
 ocamldebug buffer, then try to obtain the breakpoint info from context
 around point."
-
   (interactive "P")
   (cond
    (arg
     (let ((narg (ocamldebug-numeric-arg arg)))
       (if (> narg 0) (ocamldebug-call "delete" nil narg)
-	(with-current-buffer ocamldebug-current-buffer
-	  (if (re-search-backward "^Breakpoint [0-9]+ at [0-9]+ : file "
-				  nil t (- 1 narg))
-	      (ocamldebug-delete nil)
-	    (error "I don't have %d breakpoints in my history"
+        (with-current-buffer ocamldebug-current-buffer
+          (if (re-search-backward
+               "^Breakpoint [0-9]+ at [0-9]+\\(?::[0-9]+\\)? *: file "
+               nil t (- 1 narg))
+              (ocamldebug-delete nil)
+            (error "I don't have %d breakpoints in my history"
 		     (- 1 narg)))))))
    ((eq (current-buffer) ocamldebug-current-buffer)
-    (let* ((bpline "^Breakpoint \\([0-9]+\\) at [0-9]+ : file ")
+    (let* ((bpline
+            "^Breakpoint \\([0-9]+\\) at [0-9]+\\(?::[0-9]+\\)? *: file ")
 	   (arg (cond
 		 ((eobp)
 		  (save-excursion (re-search-backward bpline nil t))
@@ -391,7 +402,7 @@ around point."
       (with-current-buffer ocamldebug-current-buffer
 	(let ((proc (get-buffer-process (current-buffer)))
 	      (ocamldebug-filter-function #'ocamldebug-delete-filter)
-	      (ocamldebug-delete-output))
+	      ocamldebug-delete-output)
 	  (ocamldebug-call-1 "info break")
 	  (while (not (and ocamldebug-delete-output
 			   (zerop (length
@@ -434,8 +445,7 @@ around point."
                           (sort (all-completions command-word (nth 2 capf-data))
                                 #'string-lessp))))
 
-(when (fboundp 'completion-at-point)
-  (make-obsolete 'ocamldebug-complete 'completion-at-point "24.1"))
+(make-obsolete 'ocamldebug-complete 'completion-at-point "24.1")
 
 (defun ocamldebug-capf ()
   ;; FIXME: Use an `end' after point when applicable.
@@ -475,7 +485,7 @@ around point."
         nil
       ocamldebug-complete-list)))
 
-(define-key tuareg-mode-map "\C-x " 'ocamldebug-break)
+(define-key tuareg-mode-map "\C-x " #'ocamldebug-break)
 
 (defvar ocamldebug-command-name "ocamldebug"
   "Pathname for executing the OCaml debugger.")
@@ -515,14 +525,14 @@ the ocamldebug commands `cd DIR' and `directory'."
                "-emacs" "-cd" default-directory
                (append (cdr cmdlist) (cons pgm-path args)))
         (set-process-filter (get-buffer-process (current-buffer))
-                            'ocamldebug-filter)
+                            #'ocamldebug-filter)
         (set-process-sentinel (get-buffer-process (current-buffer))
-                              'ocamldebug-sentinel)
+                              #'ocamldebug-sentinel)
         (ocamldebug-mode)))
   (ocamldebug-set-buffer)))
 
 ;;;###autoload
-(defalias 'camldebug 'ocamldebug)
+(defalias 'camldebug #'ocamldebug)
 
 (defun ocamldebug-set-buffer ()
   (if (eq major-mode 'ocamldebug-mode)
@@ -534,7 +544,7 @@ the ocamldebug commands `cd DIR' and `directory'."
 (defun ocamldebug-marker-filter (string)
   (setq ocamldebug-filter-accumulator
 	(concat ocamldebug-filter-accumulator string))
-  (let ((output "") (begin))
+  (let ((output "") begin)
     ;; Process all the complete markers in this chunk.
     (while (setq begin
 		 (string-match
@@ -587,7 +597,7 @@ the ocamldebug commands `cd DIR' and `directory'."
 
 (defun ocamldebug-filter (proc string)
   (when (buffer-name (process-buffer proc))
-    (let ((process-window))
+    (let (process-window)
       (with-current-buffer (process-buffer proc)
         ;; If we have been so requested, delete the debugger prompt.
         (when (marker-buffer ocamldebug-delete-prompt-marker)
@@ -660,9 +670,9 @@ Obeying it means displaying in another window the specified file and line."
   (if (not ocamldebug-last-frame)
       (ocamldebug-remove-current-event)
     (ocamldebug-display-line (nth 0 ocamldebug-last-frame)
-                            (nth 3 ocamldebug-last-frame)
-                            (nth 4 ocamldebug-last-frame)
-                            (nth 2 ocamldebug-last-frame)))
+                             (nth 3 ocamldebug-last-frame)
+                             (nth 4 ocamldebug-last-frame)
+                             (nth 2 ocamldebug-last-frame)))
   (setq ocamldebug-last-frame-displayed-p t))
 
 ;; Make sure the file named TRUE-FILE is in a buffer that appears on the screen
@@ -670,11 +680,10 @@ Obeying it means displaying in another window the specified file and line."
 ;; Put the mark on this character in that buffer.
 
 (defun ocamldebug-display-line (true-file schar echar kind)
-  (let* ((pre-display-buffer-function nil) ; screw it, put it all in one screen
-	 (pop-up-windows t)
+  (let* ((pop-up-windows t)
 	 (buffer (find-file-noselect true-file))
 	 (window (display-buffer buffer t))
-         (spos) (epos) (pos))
+         spos epos pos)
     (with-current-buffer buffer
       (save-restriction
 	(widen)
@@ -694,7 +703,7 @@ Obeying it means displaying in another window the specified file and line."
 ;;; Events.
 
 (defun ocamldebug-remove-current-event ()
-  (if (and (fboundp 'make-overlay) window-system)
+  (if window-system
       (progn
         (delete-overlay ocamldebug-overlay-event)
         (delete-overlay ocamldebug-overlay-under))
@@ -708,7 +717,7 @@ Obeying it means displaying in another window the specified file and line."
             (move-overlay ocamldebug-overlay-under
                           (+ spos 1) epos buffer))
         (move-overlay ocamldebug-overlay-event (1- epos) epos buffer)
-        (move-overlay ocamldebug-overlay-under spos (- epos 1) buffer))
+        (move-overlay ocamldebug-overlay-under spos (1- epos) buffer))
     (with-current-buffer buffer
       (goto-char pos)
       (beginning-of-line)
@@ -734,19 +743,19 @@ Obeying it means displaying in another window the specified file and line."
              (cmd (match-string 1 str))
              (end (match-end 0))
              (subst
-              (case key
-                (?m
+              (pcase key
+                (`?m
                  (ocamldebug-module-name
                   (if insource buffer-file-name (nth 0 frame))))
-                (?d
+                (`?d
                  (file-name-directory
                   (if insource buffer-file-name (nth 0 frame))))
-                (?c
+                (`?c
                  (int-to-string
                   ;; FIXME: Should this be (- (point) (point-min))?
                   ;; What happens with multibyte chars?
                   (if insource (1- (point)) (nth 1 frame))))
-                (?e
+                (`?e
                  (save-excursion
                    (skip-chars-backward "_0-9A-Za-z\277-\377")
                    (looking-at "[_0-9A-Za-z\277-\377]*")
